package info.textgrid.lab.collatex.model.equivalences;

import info.textgrid.lab.collatex.model.text.BasicToken;
import info.textgrid.lab.collatex.model.text.BasicWitness;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementRef;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

import eu.interedition.collatex.Token;

/**
 * A token as referenced in an {@link Equivalence}.
 */
@XmlRootElement(name = "token")
public class EToken {

	@XmlTransient
	private String normalizedContent;

	@XmlTransient
	private TokenType type;

	@XmlTransient
	private Position position;

	public EToken() {
	}

	/**
	 * Creates a new {@link EToken} from the given witness token.
	 * 
	 * @param basicToken
	 *            The token in a witness.
	 */
	public EToken(final BasicToken basicToken) {
		setNormalizedContent(basicToken.getNormalized());
		setPosition(new TokenIndex(((BasicWitness) basicToken.getWitness())
				.getCollationSetWitness().getUri(), basicToken.getIndex()));
	}

	/**
	 * @see #EToken(BasicToken)
	 */
	public EToken(final Token token) {
		this((BasicToken) token);
	}

	/**
	 * The (normalized) content of the token.
	 */
	@XmlElement(required = true)
	public String getNormalizedContent() {
		return normalizedContent;
	}

	public void setNormalizedContent(final String normalizedContent) {
		this.normalizedContent = normalizedContent;
	}

	@XmlAttribute
	public TokenType getType() {
		return type;
	}

	public void setType(final TokenType type) {
		this.type = type;
	}

	/**
	 * The position of the token in its witness. This is only considered if the
	 * equivalence is local.
	 */
	@XmlElementRef
	public Position getPosition() {
		return position;
	}

	public void setPosition(final Position position) {
		this.position = position;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime
				* result
				+ ((normalizedContent == null) ? 0 : normalizedContent
						.hashCode());
		result = prime * result
				+ ((position == null) ? 0 : position.hashCode());
		result = prime * result + ((type == null) ? 0 : type.hashCode());
		return result;
	}

	@Override
	public boolean equals(final Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		final EToken other = (EToken) obj;
		if (normalizedContent == null) {
			if (other.normalizedContent != null)
				return false;
		} else if (!normalizedContent.equals(other.normalizedContent))
			return false;
		if (position == null) {
			if (other.position != null)
				return false;
		} else if (!position.equals(other.position))
			return false;
		if (type != other.type)
			return false;
		return true;
	}

	@Override
	public String toString() {
		return normalizedContent + "@" + position; //$NON-NLS-1$
	}

}
